@@ -22,6 +22,7 @@ import optparse
 import subprocess
 import tarfile
 import pipes
+import platform
 
 try:  # pragma: no cover (py2 only)
     from ConfigParser import SafeConfigParser as ConfigParser
@@ -36,7 +37,7 @@ except ImportError:  # pragma: no cover (py3 only)
 
 from pkg_resources import parse_version
 
-nodeenv_version = '0.13.5'
+nodeenv_version = '1.0.0'
 
 join = os.path.join
 abspath = os.path.abspath
@@ -46,6 +47,9 @@ is_PY3 = sys.version_info[0] == 3
 if is_PY3:
     from functools import cmp_to_key
 
+is_WIN = platform.system() == 'Windows'
+
+
 # ---------------------------------------------------------
 # Utils
 
@@ -86,7 +90,7 @@ class Config(object):
     debug = False
     profile = False
     make = 'make'
-    prebuilt = False
+    prebuilt = True
 
     @classmethod
     def _load(cls, configfiles, verbose=False):
@@ -321,7 +325,12 @@ def parse_args(check=True):
     parser.add_option(
         '--prebuilt', dest='prebuilt',
         action='store_true', default=Config.prebuilt,
-        help='Install node.js from prebuilt package')
+        help='Install node.js from prebuilt package (default)')
+
+    parser.add_option(
+        '--source', dest='prebuilt',
+        action='store_false', default=Config.prebuilt,
+        help='Install node.js from the source')
 
     options, args = parser.parse_args()
     if options.config_file is None:
@@ -366,6 +375,8 @@ def writefile(dest, content, overwrite=True, append=False):
     """
     Create file and write content in it
     """
+    mode_0755 = (stat.S_IRWXU | stat.S_IXGRP |
+                 stat.S_IRGRP | stat.S_IROTH | stat.S_IXOTH)
     content = to_utf8(content)
     if is_PY3:
         content = bytes(content, 'utf-8')
@@ -373,12 +384,13 @@ def writefile(dest, content, overwrite=True, append=False):
         logger.debug(' * Writing %s ... ', dest, extra=dict(continued=True))
         with open(dest, 'wb') as f:
             f.write(content)
+        os.chmod(dest, mode_0755)
         logger.debug('done.')
         return
     else:
         with open(dest, 'rb') as f:
             c = f.read()
-        if c == content:
+        if content in c:
             logger.debug(' * Content %s already in place', dest)
             return
 
@@ -463,14 +475,38 @@ def callit(cmd, show_stdout=True, in_shell=False,
     return proc.returncode, all_output
 
 
-def get_node_src_url(version, postfix=''):
-    node_name = '%s-v%s%s' % (get_binary_prefix(), version, postfix)
-    tar_name = '%s.tar.gz' % (node_name)
+def get_root_url(version):
     if parse_version(version) > parse_version("0.5.0"):
-        node_url = 'http://%s/dist/v%s/%s' % (src_domain, version, tar_name)
+        return 'https://%s/dist/v%s/' % (src_domain, version)
     else:
-        node_url = 'http://%s/dist/%s' % (src_domain, tar_name)
-    return node_url
+        return 'https://%s/dist/' % (src_domain)
+
+
+def get_node_bin_url(version):
+    archmap = {
+        'x86':    'x86',  # Windows Vista 32
+        'i686':   'x86',
+        'x86_64': 'x64',  # Linux Ubuntu 64
+        'AMD64':  'x64',  # Windows Server 2012 R2 (x64)
+        'armv6l': 'armv6l',     # arm
+        'armv7l': 'armv7l',
+        'aarch64': 'armv64',
+    }
+    sysinfo = {
+        'system': platform.system().lower(),
+        'arch': archmap[platform.machine()],
+    }
+    if is_WIN:
+        filename = 'win-%(arch)s/node.exe' % sysinfo
+    else:
+        postfix = '-%(system)s-%(arch)s.tar.gz' % sysinfo
+        filename = '%s-v%s%s' % (get_binary_prefix(), version, postfix)
+    return get_root_url(version) + filename
+
+
+def get_node_src_url(version):
+    tar_name = '%s-v%s.tar.gz' % (get_binary_prefix(), version)
+    return get_root_url(version) + tar_name
 
 
 @contextlib.contextmanager
@@ -483,25 +519,23 @@ def tarfile_open(*args, **kwargs):
         tf.close()
 
 
-def download_node(node_url, src_dir, env_dir, opt):
+def download_node_src(node_url, src_dir, opt, prefix):
     """
     Download source code
     """
+    logger.info('.', extra=dict(continued=True))
     tar_contents = io.BytesIO(urlopen(node_url).read())
+    logger.info('.', extra=dict(continued=True))
     with tarfile_open(fileobj=tar_contents) as tarfile_obj:
-        tarfile_obj.extractall(src_dir)
-    logger.info(')', extra=dict(continued=True))
-
-
-def get_node_src_url_postfix(opt):
-    if not opt.prebuilt:
-        return ''
-
-    import platform
-    postfix_system = platform.system().lower()
-    arches = {'x86_64': 'x64', 'i686': 'x86'}
-    postfix_arch = arches[platform.machine()]
-    return '-{0}-{1}'.format(postfix_system, postfix_arch)
+        member_list = tarfile_obj.getmembers()
+        extract_list = []
+        for member in member_list:
+            node_ver = opt.node.replace('.', '\.')
+            regex_string = "%s-v%s[^/]*/(README\.md|CHANGELOG\.md|LICENSE)" \
+                % (prefix, node_ver)
+            if re.match(regex_string, member.name) is None:
+                extract_list.append(member)
+        tarfile_obj.extractall(src_dir, extract_list)
 
 
 def urlopen(url):
@@ -588,17 +622,22 @@ def install_node(env_dir, src_dir, opt):
     Download source code for node.js, unpack it
     and install it in virtual environment.
     """
+    env_dir = abspath(env_dir)
     prefix = get_binary_prefix()
-    logger.info(' * Install %s (%s' % (prefix, opt.node),
+    node_src_dir = join(src_dir, to_utf8('%s-v%s' % (prefix, opt.node)))
+    src_type = "prebuilt" if opt.prebuilt else "source"
+
+    logger.info(' * Install %s %s (%s) ' % (src_type, prefix, opt.node),
                 extra=dict(continued=True))
 
-    node_url = get_node_src_url(opt.node, get_node_src_url_postfix(opt))
-    node_src_dir = join(src_dir, to_utf8('%s-v%s' % (prefix, opt.node)))
-    env_dir = abspath(env_dir)
+    if opt.prebuilt:
+        node_url = get_node_bin_url(opt.node)
+    else:
+        node_url = get_node_src_url(opt.node)
 
     # get src if not downloaded yet
     if not os.path.exists(node_src_dir):
-        download_node(node_url, src_dir, env_dir, opt)
+        download_node_src(node_url, src_dir, opt, prefix)
 
     logger.info('.', extra=dict(continued=True))
 
@@ -678,8 +717,6 @@ def install_activate(env_dir, opt):
     bin_dir = join(env_dir, 'bin')
     mod_dir = join('lib', 'node_modules')
     prompt = opt.prompt or '(%s)' % os.path.basename(os.path.abspath(env_dir))
-    mode_0755 = (stat.S_IRWXU | stat.S_IXGRP |
-                 stat.S_IRGRP | stat.S_IROTH | stat.S_IXOTH)
 
     shim_node = join(bin_dir, "node")
     shim_nodejs = join(bin_dir, "nodejs")
@@ -711,7 +748,6 @@ def install_activate(env_dir, opt):
         # existing python's virtual environment
         need_append = 0 if name in ('node', 'shim') else opt.python_virtualenv
         writefile(file_path, content, append=need_append)
-        os.chmod(file_path, mode_0755)
 
     if not os.path.exists(shim_nodejs):
         os.symlink("node", shim_nodejs)
@@ -988,7 +1024,7 @@ if [ "${BASH_SOURCE}" ] ; then
     SOURCE="${BASH_SOURCE[0]}"
 
     while [ -h "$SOURCE" ] ; do SOURCE="$(readlink "$SOURCE")"; done
-    DIR="$( cd -P "$( dirname "$SOURCE" )" && pwd )"
+    DIR="$( command cd -P "$( dirname "$SOURCE" )" > /dev/null && pwd )"
 
     NODE_VIRTUAL_ENV="$(dirname "$DIR")"
 else
@@ -1001,7 +1037,7 @@ fi
 export NODE_VIRTUAL_ENV
 
 _OLD_NODE_VIRTUAL_PATH="$PATH"
-PATH="$NODE_VIRTUAL_ENV/__BIN_NAME__:$PATH"
+PATH="$NODE_VIRTUAL_ENV/lib/node_modules/.bin:$NODE_VIRTUAL_ENV/__BIN_NAME__:$PATH"
 export PATH
 
 _OLD_NODE_PATH="$NODE_PATH"
